//---------------------------------------------------------------------------
// Sync.cpp
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Sync.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Sync {
//---------------------------------------------------------------------------
void __fastcall TMutex::Initialize(const AnsiString& AName, bool ALock)
{
	try {
		Create(AName, ALock);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Uninitialize()
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Create(const AnsiString& AName, bool ALock)
{
	::SetLastError(NO_ERROR);
	HANDLE newHandle = ::CreateMutex(NULL, ALock, AName.c_str());
	if(newHandle == NULL) {
		throw ESyncError("The program cannot create a mutex object.");
	}
	FHandle = newHandle;
	if(ALock && ::GetLastError() != ERROR_ALREADY_EXISTS) {
		FLockCount++;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Close()
{
	if(Handle == NULL) {
		return;
	}
	while(FLockCount > 0) {
		Unlock();
	}
	::CloseHandle(FHandle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
__fastcall TMutex::TMutex(const AnsiString& AName, bool ALock) :
	FHandle(NULL),
	FLockCount(0)
{
	Initialize(AName, ALock);
}
//---------------------------------------------------------------------------
__fastcall TMutex::~TMutex()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Lock(int ATimeoutInterval)
{
	if(!TryLock(ATimeoutInterval)) {
		throw ESyncError("The program cannot lock the mutex object.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TMutex::TryLock(int ATimeoutInterval)
{
	switch(::WaitForSingleObject(Handle, ATimeoutInterval < 0 ? INFINITE : ATimeoutInterval)) {
	case WAIT_ABANDONED:
	case WAIT_OBJECT_0:
		FLockCount++;
		return true;
	case WAIT_TIMEOUT:
		return false;
	case WAIT_FAILED:
	default:
		throw ESyncError("The program cannot wait for the mutex object.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Unlock()
{
	if(FLockCount == 0) {
		return;
	}
	::ReleaseMutex(Handle);
	FLockCount--;
}
//---------------------------------------------------------------------------
void __fastcall TCriticalSection::Initialize()
{
	try {
		::InitializeCriticalSection(Value);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TCriticalSection::Uninitialize()
{
	while(FLockCount > 0) {
		Unlock();
	}
	::DeleteCriticalSection(Value);
}
//---------------------------------------------------------------------------
CRITICAL_SECTION* __fastcall TCriticalSection::GetValue()
{
	return &FValue;
}
//---------------------------------------------------------------------------
__fastcall TCriticalSection::TCriticalSection() :
	FValue(),
	FLockCount(0)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TCriticalSection::~TCriticalSection()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TCriticalSection::Lock()
{
	::EnterCriticalSection(Value);
	FLockCount++;
}
//---------------------------------------------------------------------------
void __fastcall TCriticalSection::Unlock()
{
	if(FLockCount == 0) {
		return;
	}
	::LeaveCriticalSection(Value);
	FLockCount--;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Sync
//---------------------------------------------------------------------------

