//---------------------------------------------------------------------------
// MainFormUnit.cpp
//
// Version: 1.0.0 +
// Last modified: 2012-01-19 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "MainFormUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm* MainForm = NULL;
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	try {
		SettingsHandler->Save();
		WheelHook->Enabled = false; // Ensure the hook is disabled
	}
	catch(ESettingsHandlerError& e) {
		Application->ShowException(&e);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ActionList1Update(TBasicAction* AAction, bool& AHandled)
{
	AHandled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainFormActivateApplicationAcExecute(TObject* ASender)
{
	TryForceSetForegroundWindow(::GetLastActivePopup(Application->Handle));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainFormShowMenuAcExecute(TObject* ASender)
{
	TryForceSetForegroundWindow(::GetLastActivePopup(Application->Handle));
	if(::GetLastActivePopup(Application->Handle) != Application->Handle) {
		return;
	}
	POINT menuPos = {0};
	if(!::GetCursorPos(&menuPos)) {
		menuPos.x = menuPos.y = 0;
	}
	::SetForegroundWindow(NotificationIcon->Handle);
	PopupMenu1->Popup(menuPos.x, menuPos.y);
	::PostMessage(NotificationIcon->Handle, WM_NULL, 0, 0); // Prevent the menu from disappearing immediately
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainFormShowOptionsAcExecute(TObject* ASender)
{
	TryForceSetForegroundWindow(::GetLastActivePopup(Application->Handle));
	if(::GetLastActivePopup(Application->Handle) != Application->Handle) {
		return;
	}
	SettingsShowOptionsAc->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainFormExitAcExecute(TObject* ASender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SettingsShowOptionsAcExecute(TObject* ASender)
{
	SettingsHandler->ShowOptions();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpShowContentsAcExecute(TObject* ASender)
{
	HelpHandler->ShowContents(true);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpShowAboutAcExecute(TObject* ASender)
{
	HelpHandler->ShowAbout();
}
//---------------------------------------------------------------------------
const UINT TMainForm::WM_MULTIPLEAPPINSTANCES = ::RegisterWindowMessage("Moochos.WheelHere.MainForm.WM_MULTIPLEAPPINSTANCES");
//---------------------------------------------------------------------------
void __fastcall TMainForm::Initialize()
{
	try {
		FHelpHandler = new THelpHandler();
		FSettingsHandler = new TSettingsHandler();

		FWheelHook = new TWheelHook();
		FWheelHook->OnChange = WheelHookChange;

		FNotificationIcon = new TNotificationIcon(0);
		FNotificationIcon->Hint = Application->Title;
		FNotificationIcon->ShowHint = true;
		FNotificationIcon->OnMessage = NotificationIconMessage;

		FProcessMessenger = new TMessageWindow();
		FProcessMessenger->OnMessage = ProcessMessengerMessage;

		WheelHook->OnChange(WheelHook);

		SettingsHandler->Reset();
		SettingsHandler->Load();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Uninitialize()
{
	delete FProcessMessenger;
	FProcessMessenger = NULL;

	delete FNotificationIcon;
	FNotificationIcon = NULL;

	delete FWheelHook;
	FWheelHook = NULL;

	delete FSettingsHandler;
	FSettingsHandler = NULL;

	delete FHelpHandler;
	FHelpHandler = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WheelHookChange(TObject* ASender)
{
	ImageList1->GetIcon(WheelHook->Enabled ? 0 : 1, NotificationIcon->Icon);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NotificationIconMessage(TObject* ASender, UINT AKind)
{
	if(AKind == WM_LBUTTONDBLCLK) {
		MainFormShowOptionsAc->Execute();
	}
	else if(AKind == WM_LBUTTONUP) {
		MainFormActivateApplicationAc->Execute();
	}
	else if(AKind == WM_RBUTTONUP) {
		MainFormShowMenuAc->Execute();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessMessengerMessage(TObject* ASender, TMessage& AMessage)
{
	if(AMessage.Msg == WM_MULTIPLEAPPINSTANCES) {
		MainFormShowOptionsAc->Execute();
	}
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FHelpHandler(NULL),
	FSettingsHandler(NULL),
	FWheelHook(NULL),
	FNotificationIcon(NULL),
	FProcessMessenger(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

