//---------------------------------------------------------------------------
// SettingsHandler.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/IniFiles.h"
#include "MainFormUnit.h"
#include "OptionsDialogUnit.h"
#include "SettingsHandler.h"
#include "WheelHook.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace WheelHere { namespace SettingsHandler {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString SettingsFileName = ExtractFilePath(Application->ExeName) + "Settings.ini";

const AnsiString keyDisplayIcon = "DisplayIcon";
const AnsiString keyEnabled = "Enabled";
const AnsiString keyFocus = "Focus";
const AnsiString keyMouseWheel = "MouseWheel";
const AnsiString keyScroll = "Scroll";
const AnsiString keyView = "View";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
TSettings __fastcall TSettingsHandler::LoadFromFile(TVarIniFile* AIniFile)
{
	TSettings settings = Settings;
	settings.MouseWheel.Enabled = AIniFile->ReadBool(
		keyMouseWheel,
		keyEnabled,
		settings.MouseWheel.Enabled);
	settings.MouseWheel.Focus = AIniFile->ReadBool(
		keyMouseWheel,
		keyFocus,
		settings.MouseWheel.Focus);
	settings.MouseWheel.Scroll = AIniFile->ReadBool(
		keyMouseWheel,
		keyScroll,
		settings.MouseWheel.Scroll);
	settings.View.DisplayIcon = AIniFile->ReadBool(
		keyView,
		keyDisplayIcon,
		settings.View.DisplayIcon);
	return settings;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SaveToFile(TVarIniFile* AIniFile, const TSettings& ASettings)
{
	AIniFile->WriteBool(
		keyMouseWheel,
		keyEnabled,
		ASettings.MouseWheel.Enabled);
	AIniFile->WriteBool(
		keyMouseWheel,
		keyFocus,
		ASettings.MouseWheel.Focus);
	AIniFile->WriteBool(
		keyMouseWheel,
		keyScroll,
		ASettings.MouseWheel.Scroll);
	AIniFile->WriteBool(
		keyView,
		keyDisplayIcon,
		ASettings.View.DisplayIcon);
	AIniFile->UpdateFile();
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::OptionsDialogApply(TObject* ASender)
{
	Options = dynamic_cast<TOptionsDialog*>(ASender)->Value;
}
//---------------------------------------------------------------------------
TSettings __fastcall TSettingsHandler::GetSettings()
{
	return GetSettingsAs(true);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SetSettings(const TSettings& ASettings)
{
	SetSettingsAs(ASettings, true);
}
//---------------------------------------------------------------------------
TSettings __fastcall TSettingsHandler::GetOptions()
{
	return GetSettingsAs(false);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SetOptions(const TSettings& AOptions)
{
	SetSettingsAs(AOptions, false);
}
//---------------------------------------------------------------------------
TSettings __fastcall TSettingsHandler::GetSettingsAs(bool AAsSettings)
{
	TSettings settings;
	{
		settings.MouseWheel.Enabled = MainForm->WheelHook->Enabled;

		Wheeler_TSettings wheeler_Settings = MainForm->WheelHook->Settings;
		settings.MouseWheel.Focus = wheeler_Settings.Focus;
		settings.MouseWheel.Scroll = wheeler_Settings.Scroll;
	}
	{
		settings.View.DisplayIcon = MainForm->NotificationIcon->Visible;
	}
	return settings;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SetSettingsAs(const TSettings& ASettings, bool AAsSettings)
{
	{
		MainForm->WheelHook->Enabled = ASettings.MouseWheel.Enabled;

		Wheeler_TSettings wheeler_Settings = MainForm->WheelHook->Settings;
		wheeler_Settings.Focus = ASettings.MouseWheel.Focus;
		wheeler_Settings.Scroll = ASettings.MouseWheel.Scroll;
		MainForm->WheelHook->Settings = wheeler_Settings;
	}
	{
		MainForm->NotificationIcon->Visible = ASettings.View.DisplayIcon;
	}
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandler::TSettingsHandler() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandler::~TSettingsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Load()
{
	try {
		if(!FileExists(SettingsFileName)) {
			return;
		}
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		Settings = LoadFromFile(iniFile);
	}
	catch(EIniFileException& e) {
		throw ESettingsHandlerError("The program cannot load the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Save()
{
	try {
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		SaveToFile(iniFile, Settings);
	}
	catch(EIniFileException& e) {
		throw ESettingsHandlerError("The program cannot save the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Reset()
{
	try {
		TSettings newSettings = Settings;
		newSettings.MouseWheel.Enabled = true;
		newSettings.MouseWheel.Focus = true;
		newSettings.MouseWheel.Scroll = true;
		newSettings.View.DisplayIcon = true;
		Settings = newSettings;
	}
	catch(EIniFileException& e) {
		throw ESettingsHandlerError("The program cannot reset the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::ShowOptions()
{
	TPtr<TOptionsDialog> optionsDialog = new TOptionsDialog(NULL);
	optionsDialog->Value = Options;
	optionsDialog->OnApply = OptionsDialogApply;
	if(!optionsDialog->Execute()) {
		Abort();
	}
	Options = optionsDialog->Value;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::WheelHere::SettingsHandler
//---------------------------------------------------------------------------

