//---------------------------------------------------------------------------
// WheelHook.cpp
//
// Version: 1.0.1
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "WheelHook.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace WheelHere { namespace WheelHook {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString WheelerFileName = "Wheeler.dll";
const AnsiString msgCannotGetProp = "The program cannot get the property.";
const AnsiString msgCannotSetProp = "The program cannot set the property.";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TWheelHook::Initialize()
{
	try {
		if(
			(FWheeler = ::LoadLibrary(WheelerFileName.c_str())) == NULL ||

			(FInner_Wheeler_IsEnabled = reinterpret_cast<TInner_Wheeler_IsEnabled>(
				::GetProcAddress(FWheeler, "Wheeler_IsEnabled"))) == NULL ||

			(FInner_Wheeler_SetEnabled = reinterpret_cast<TInner_Wheeler_SetEnabled>(
				::GetProcAddress(FWheeler, "Wheeler_SetEnabled"))) == NULL ||

			(FInner_Wheeler_GetSettings = reinterpret_cast<TInner_Wheeler_GetSettings>(
				::GetProcAddress(FWheeler, "Wheeler_GetSettings"))) == NULL ||

			(FInner_Wheeler_SetSettings = reinterpret_cast<TInner_Wheeler_SetSettings>(
				::GetProcAddress(FWheeler, "Wheeler_SetSettings"))) == NULL
			) {
			throw EWheelHookError(AnsiString().sprintf(
				"The program cannot load the library \"%s\".",
				WheelerFileName.c_str()));
		}
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TWheelHook::Uninitialize()
{
	if(FWheeler != NULL) {
		::FreeLibrary(FWheeler);
		FWheeler = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TWheelHook::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TWheelHook::GetEnabled()
{
	bool enabled = false;
	if(!FInner_Wheeler_IsEnabled(enabled)) {
		throw EWheelHookError(msgCannotGetProp);
	}
	return enabled;
}
//---------------------------------------------------------------------------
void __fastcall TWheelHook::SetEnabled(bool AEnabled)
{
	if(!FInner_Wheeler_SetEnabled(AEnabled)) {
		throw EWheelHookError(msgCannotSetProp);
	}
	DoChange();
}
//---------------------------------------------------------------------------
Wheeler_TSettings __fastcall TWheelHook::GetSettings()
{
	Wheeler_TSettings settings;
	if(!FInner_Wheeler_GetSettings(settings)) {
		throw EWheelHookError(msgCannotGetProp);
	}
	return settings;
}
//---------------------------------------------------------------------------
void __fastcall TWheelHook::SetSettings(const Wheeler_TSettings& ASettings)
{
	if(!FInner_Wheeler_SetSettings(ASettings)) {
		throw EWheelHookError(msgCannotSetProp);
	}
	DoChange();
}
//---------------------------------------------------------------------------
__fastcall TWheelHook::TWheelHook() :
	TObject(),
	FWheeler(NULL),
	FInner_Wheeler_IsEnabled(NULL),
	FInner_Wheeler_SetEnabled(NULL),
	FInner_Wheeler_GetSettings(NULL),
	FInner_Wheeler_SetSettings(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TWheelHook::~TWheelHook()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::WheelHere::WheelHook
using namespace Moochos::WheelHere::WheelHook;
//---------------------------------------------------------------------------

